'use client'

import { motion, MotionValue, useTime, useTransform } from 'framer-motion';
import { useMemo } from 'react';

const titles = [
    'Full Stack Developer',
    'DevOps Engineer',
    'Game Developer',
    'Open Source Contributor',
    'Tech Enthusiast',
    'Gamer',
];

function Title({ title, progress, range, ys, os }: { title: string, progress: MotionValue<number>, range: number[], ys: string[], os: number[] }) {
    const y = useTransform(progress, range, ys);
    const opacity = useTransform(progress, range, os);

    return (
        <motion.div
            className="absolute w-full text-center text-4xl font-bold"
            style={{
                y,
                opacity
            }}
        >
            {title}
        </motion.div>
    );
}

export default function Slide1() {
    const time = useTime();
    


    const genRanges = useMemo(() => {
        const ranges = [];
        for (let i = 0; i < titles.length+1; i++) {
            const start = i * 1500 + 100 * i;
            const end = start + 1500;
            ranges.push(start, end);
        }
        return ranges;
    }, []);

    const genYs = useMemo(() => {
        const ys = [];
        for (let i = 0; i < titles.length; i++) {
            ys.push([] as string[]);
            for (let j = 0; j < titles.length+1; j++) {
                if ((j % titles.length) === i) {
                    ys[i].push('0%', '0%'); // Current title
                } else if (j < i) {
                    ys[i].push('100%', '100%'); // Previous titles
                } else if (j == i + 1) {
                    ys[i].push('-100%', '100%'); // Next title
                } else {
                    ys[i].push('100%', '100%'); // Titles after next
                }
            }
        }
        return ys;
    }, []);

    const genOs = useMemo(() => {
        const os = [];
        for (let i = 0; i < titles.length; i++) {
            os.push([] as number[]);
            for (let j = 0; j < titles.length+1; j++) {
                if ((j % titles.length) === i) {
                    os[i].push(1, 1); // Current title
                } else if (j < i) {
                    os[i].push(0, 0); // Previous titles
                } else if (j == i + 1) {
                    os[i].push(0, 0); // Next title
                } else {
                    os[i].push(0, 0); // Titles after next
                }
            }
        }
        return os;
    }, []);

    const progress = useTransform(time, (t) => ((t * 1) % genRanges[genRanges.length - 2]));

    //

    return (
        <>
            <p>I&apos;m Kay</p>
            <div className="relative flex flex-col h-10 w-full overflow-visible">
                {titles.map((title, i) => (
                    <Title
                        key={i}
                        title={title}
                        progress={progress}
                        range={genRanges}
                        ys={genYs[i]}
                        os={genOs[i]}
                    />
                ))}
            </div>
        </>
    )
}